#  tests for navigator-updater-0.4.0-py310h06a4308_0 (this is a generated file);
print('===== testing package: navigator-updater-0.4.0-py310h06a4308_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import navigator_updater

print('navigator_updater.__version__: %s' % navigator_updater.__version__)
assert navigator_updater.__version__ == '0.4.0'
#  --- run_test.py (end) ---

print('===== navigator-updater-0.4.0-py310h06a4308_0 OK =====');
print("import: 'navigator_updater'")
import navigator_updater

